/*
 * 22:10:08 09/04/99
 *
 * Player.java - Extends Ship.class
 * Copyright (C) 1999 Romain Guy
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.chez.powerteam.retaliator;

import java.awt.Image;
import java.awt.Dimension;
import java.awt.image.ImageObserver;

/**
 * The Player class is used to manage a player. This class can be used
 * in any kind of game because it implements standards properties such
 * as score, lives...
 * @author Romain Guy
 * @version 1.0
 * @see Laser
 */

public class Player extends Ship
{
  private int lives;
  private int score;

  /**
   * The constructor is like the Ship's one but needs a supplementary <code>int</code>.
   * This value is used to determine how many lives the Player starts with.
   * @param lives The amount of lives when Player begin
   * @param img The Image used to represent one Player
   * @param width The width of the area where the Player will be painted
   * @param height The height of the area where the Player will be painted
   * @param o The ImageObserver used to paint the Player
   */

  public Player(int lives, Image img, int width, int height, ImageObserver o)
  {
    super(img, width, height, o);
    this.lives = lives;
  }

  /**
   * Returns remaining quantity of lives.
   */

  public int getLives()
  {
    return lives;
  }

  /**
   * Set the amount of Player's life.
   * @param amount The new Player's amount of lives
   */

  public void setLives(int amount)
  {
    if (amount >= 0) lives = amount;
  }

  /**
   * Remove a life to the Player. It is used when the Player... dies !
   */

  public void removeLife()
  {
    if (--lives < 0) lives = 0;
  }

  /**
   * Add a life to the Player. Call it when Player get a bonus or a
   * specific score.
   */

  public void addLife()
  {
    lives++;
  }

  /**
   * Return the Player's score.
   */

  public int getScore()
  {
    return score;
  }

  /**
   * You may need to set the score of the Player. For example, when you start
   * a new game.
   * @param score The new Player's score
   */

  public void setScore(int score)
  {
    this.score = score;
  }

  /**
   * When Player pick up an object or kills an ennemy, you'll have to increase
   * his score. So, instead of calling <code>setScore(int)</code>, use this.
   * @param bonus The bonus to be added to the Player's current score
   */

  public void increaseScore(int bonus)
  {
    score += bonus;
  }

  /**
   * You may need to decrease score of the Player, for example when he
   * kills an hostage or dies. Better than <code>setScore(int)</code>.
   * @param malus The malus to be removed from the Player's current score
   */

  public void decreaseScore(int malus)
  {
    score -= malus;
  }

  /**
   * When you start a new game, calling this method will reset the score
   * to 0. Still better than <code>setScore(int)</code>.
   */

  public void resetScore()
  {
    score = 0;
  }
}

// End of Player.java
