/*
 * =============================================================================================
 * Retaliator - a Galaga clone -        v0.4.3.2 alpha (free)        18:13:56 12/06/99
 * =============================================================================================
 *
 * Retaliator.java - A shoot'em up
 * Copyright (C) 1999 Romain Guy
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.chez.powerteam.retaliator;

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import java.applet.AudioClip;

/**
 * Retaliator is a shoot'em up game taking place in the Star War's universe
 * a lot of time later. This shoot'em up has been designed for Web pages and
 * so, it is easy to play, fast to load and funny. Hope you'll like it !
 * @author Romain Guy
 * @version 0.4.3.2
 */

public class Retaliator extends Applet implements Runnable
{
  // ---- Optimized display (Double Buffering) ----

  private Thread runner;
  private Graphics off;
  private Image offImg;
  private MediaTracker tracker;

  // ---- Player's ship ----

  private Player xwing;
  private StarField back;
  private AudioClip weapon;
  private Image life, xImg, laserImg;
  private Laser laserLeft, laserRight;

  private int oldPositionX;

  // ---- Surfaces constants ----

  private int width;
  private int height;

  // ---- Booleans ----

  private boolean laserFired;
  private boolean endOfGame;
  private boolean intro = true;

  // ---- Introduction ----

  private Image introImg;
  private Point introCenter;

  // ---- Used to display the score ----

  private int fontWidth;

  /**
   * Initialize the game. Add <code>listeners</code>, prepare images,
   * create instances of <code> Player, Laser, StarField</code>.
   */

  public void init()
  {
    addMouseListener(new RetaliatorListener(this));
    addMouseMotionListener(new RetaliatorListener(this));

    // ---- Surface ----

    // Deprecated methods
    // Ensure compatibility
    width = size().width;
    height = size().height;

    // I'd better have to use (since JDK 1.1):
    //   width = getWidth();
    //   height = getHeight();

    // ---- Double Buffering's Initialisation ----

    setBackground(Color.black);
    offImg = createImage(width, height);
    off = offImg.getGraphics();

    // ---- Preparing images ----

    tracker = new MediaTracker(this);
    introImg = getImage(getClass().getResource("gfx/intro.gif"));
    tracker.addImage(introImg, 0);
    xImg = getImage(getClass().getResource("gfx/xwing.gif"));
    tracker.addImage(xImg, 1);
    life = getImage(getClass().getResource("gfx/life.gif"));
    tracker.addImage(life, 2);
    laserImg = getImage(getClass().getResource("gfx/laser.gif"));
    tracker.addImage(laserImg, 3);

    // ---- Waiting for image loading ----

    try
    {
      tracker.waitForAll();
    } catch (InterruptedException e) { stop(); }
    if (tracker.isErrorAny())
      stop();

    // ---- Ship's Initialisation ----

    xwing = new Player(5, xImg, width, height, this);
    xwing.setPosition(width / 2 + 16, height - 32, Ship.NO_CLIP);
    laserLeft = new Laser(1, laserImg, width, height, this);
    laserLeft.setPosition(width / 2 + 16, height - 32, Ship.NO_CLIP);
    laserRight = new Laser(1, laserImg, width, height, this);
    laserRight.setPosition(width / 2 + 29, height - 32, Ship.NO_CLIP);
    weapon = getAudioClip(getClass().getResource("sound/laser.au"));

    oldPositionX = xwing.getPositionX();

    introCenter = new Point((width - introImg.getWidth(this)) / 2, (height - introImg.getHeight(this)) / 2);

    // ---- Create a StarField ----

    int colors[] = { 105, 123, 145 };
    int moves[] = { 3, 2, 1 };
    back = new StarField(colors, moves, 10, StarField.VERTICAL, width, height);

    // ---- Used to display the score ----

    fontWidth = getToolkit().getFontMetrics(this.getFont()).charWidth('m');
  }

  /**
   * Start Retaliator's Thread.
   */

  public void start()
  {
    // ---- Start the thread ----

    if (runner == null)
    {
      runner = new Thread(this);
      runner.start();
    }
  }

  /**
   * Called when the Thread stop. Also stop the game.
   */

  public void stop()
  {
    if (getIntroState()) intro = false;
    endOfGame = true;
    runner = null;
  }

  /**
   * On browser closing, we <code>free()</code> each instance of
   * <code>Ship</code> and <code>flush()</code> the <code>Graphics</code>.
   */

  public void destroy()
  {
    // ---- Empty memory ----

    xImg.flush();
    xImg = null;
    laserImg.flush();
    laserImg = null;

    xwing.free();
    laserLeft.free();
    laserRight.free();
    back.free();

    offImg.flush();
    offImg = null;
    off.dispose();
  }

  /**
   * Run the game. First display the introduction.
   */

  public void run()
  {
    // ---- Introduction ----

    while (getIntroState())
      paintIntro();
    introImg.flush();

    // ---- Game ----

    while (!isEndOfGame())
    {
      paintGame();
      runner.yield();
    }
  }

  /**
   * Return true if the introduction is displaying.
   */

  public boolean getIntroState()
  {
    return intro;
  }

  /**
   * Return true if the Player has already fired.
   */

  public boolean getFireState()
  {
    return laserFired;
  }

  /**
   * Return true if game has ended.
   */

  public boolean isEndOfGame()
  {
    return endOfGame;
  }

  /**
   * For the same reason as for <code>repaint()</code>, I've overrided
   * this method.
   * @param g The Graphics to be updated
   */

  public void update(Graphics g)
  {
    off.setColor(Color.black);
    off.fillRect(0, 0, width, height);
  }

  /**
   * Drop a laser burst.
   */

  public void fire()
  {
    if (!getFireState() && !getIntroState())
    {
      weapon.play();
      laserFired = true;
    } else if (getIntroState())
      intro = false;
  }

  /**
   * Move the player's ship.
   * @param evt A MouseEvent is needed to determine new ship's position
   */

  public void moveXWing(MouseEvent evt)
  {
    if (evt.getX() == oldPositionX) return;
    xwing.setPosition(evt.getX(), height - 32, Ship.BOUNDARY_SCREEN);
    if (!getFireState())
    {
      laserLeft.setPosition(xwing.getPositionX(), height - 40, Ship.NO_CLIP);
      laserRight.setPosition(xwing.getPositionX() + 29, height - 40, Ship.NO_CLIP);
    }
    oldPositionX = xwing.getPositionX();
    repaint();
  }

  /**
   * Test if a new laser can be fired or not.
   */

  public void laserTest()
  {
    if (getFireState())
    {
      laserLeft.paint(off, this);
      laserLeft.move(0, -3, Ship.NO_CLIP);
      laserRight.paint(off, this);
      laserRight.move(0, -3, Ship.NO_CLIP);
      if (laserLeft.getPositionY() < -30)
      {
        laserFired = false;
	laserLeft.setPosition(xwing.getPositionX(), height - 40, Ship.NO_CLIP);
	laserRight.setPosition(xwing.getPositionX() + 29, height - 40, Ship.NO_CLIP);
      }
    }
  }

  /**
   * Paint the player score and the amount of remaining lives.
   */

  public synchronized void paintPlayerStatus()
  {
    off.drawImage(life, 0, height - 16, this);
    off.setColor(Color.yellow);
    off.drawString(String.valueOf(xwing.getLives()), 22, height);
    String score = String.valueOf(xwing.getScore());
    off.drawString(score, width - (fontWidth * score.length()), height);
  }

  /**
   * Paint the introduction.
   */

  public synchronized void paintIntro()
  {
    update(getGraphics());
    if (!back.isDrawing()) back.paint(off);
    off.drawImage(introImg, introCenter.x, introCenter.y, this);
    paint(getGraphics());
  }

  /**
   * Paint the game.
   */

  public synchronized void paintGame()
  {
    update(getGraphics());
    if (!back.isDrawing()) back.paint(off);
    laserTest();
    xwing.paint(off, this);
    paintPlayerStatus();
    paint(getGraphics());
  }

  /**
   * Paint the starfield, the player and the ennemies.
   * @param g The Graphics where to paint all those stuffs
   */

  public void paint(Graphics g)
  {
    g.drawImage(offImg, 0, 0, this);
  }

  /**
   * Returns information about this applet (copyright...).
   */

  public String getAppletInfo()
  {
    return "Retaliator\n(C)1998-1999 Romain Guy\n\npowerteam@chez.com\nwww.chez.com/powerteam\n\nICQ: #36137571";
  }
}

// End of Retaliator.java
