/*
 * 18:16:14 11/04/99
 *
 * RetaliatorListener.java - Handles Retaliator's mouse input
 * Copyright (C) 1999 Romain Guy
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.chez.powerteam.retaliator;

import java.awt.event.*;

/**
 * The RetaliatorListener class handles all the mouse
 * inputs events for Retaliator. This makes code lighter
 * and easier to manage.
 * @author Romain Guy
 * @version 1.0
 * @see Retaliator
 */

public class RetaliatorListener implements MouseListener, MouseMotionListener
{
  private Retaliator parent;

  /**
   * This constructor is necessary to get the Retaliator's instance
   * used by the navigator. Otherwise, we couldn't be able to call
   * Retaliator's methods such as <code>fire()</code>.
   * @param parent The instance of Retaliator's class which calls this constructor
   */

  public RetaliatorListener(Retaliator parent)
  {
    this.parent = parent;
  }

  /**
   * Call the <code>fire()</code> method to make the Player
   * dropping a laser burst.
   */

  public void mousePressed(MouseEvent evt)
  {
    parent.fire();
  }

  public void mouseReleased(MouseEvent evt) { }

  /**
   * Display my copyright.
   */

  public void mouseEntered(MouseEvent evt)
  {
    parent.getAppletContext().showStatus("Retaliator (c)1999 Romain Guy");
  }

  /**
   * Display my copyright.
   */

  public void mouseExited(MouseEvent evt)
  {
    parent.getAppletContext().showStatus("Retaliator (c)1999 Romain Guy");
  }

  public void mouseClicked(MouseEvent evt) { }

  /**
   * The Player must move even if the user hasn't released the mouse
   * button since his last shot.
   */

  public void mouseDragged(MouseEvent evt)
  {
    if (!parent.getIntroState()) parent.moveXWing(evt);
  }

  /**
   * Move the player.
   */

  public void mouseMoved(MouseEvent evt)
  {
    if (!parent.getIntroState()) parent.moveXWing(evt);
  }
}

// End of RetaliatorListener.java
